/* 
 * PLOP sample application for dumping PDF information with PLOP functions
 */

import java.io.*;
import com.pdflib.plop;
import com.pdflib.PLOPException;

public class dumper
{
    static final String	docoptlist = "requiredmode=minimum";
    static final String	globaloptlist = "";

    static plop plop = null;

    /* This is where input files live. Adjust as necessary. */
    static final String searchpath = "../data";

    public static void main (String argv[])
    {
	int	count, pcosmode, recipientcount;
	int	i, doc;
	boolean plainmetadata;
	String	objtype;
	String	optlist;

	try
	{
	    if (argv.length != 1)
	    {   
		throw new Exception("usage: dumper <filename>");
	    }

	    plop = new plop();

	    optlist = "searchpath={" + searchpath + "}";
	    plop.set_option(optlist);

	    plop.set_option(globaloptlist);

	    if ((doc = plop.open_document(argv[0], docoptlist)) == -1)
	    {
		throw new Exception("ERROR " + plop.get_errmsg());
	    }

	    /* --------- general information (always available) */

	    pcosmode = (int) plop.pcos_get_number(doc, "pcosmode");

	    System.out.println("       File name: " + plop.pcos_get_string(doc,"filename"));

	    System.out.println("     PDF version: " +
		plop.pcos_get_string(doc, "pdfversionstring"));

	    System.out.println("      Encryption: " +
		plop.pcos_get_string(doc, "encrypt/description"));
	    recipientcount =
		    (int) plop.pcos_get_number(doc, "length:encrypt/recipients");

	    if (recipientcount > 0)
	    {
		System.out.println("Recipient groups: " + recipientcount);
		System.out.print("      Digital ID: ");

		if (pcosmode == 2)
		    System.out.println("supplied digital ID has master permissions");
		else if (pcosmode == 1)
		    System.out.println("supplied digital ID doesn't have master permissions");
		else
		    System.out.println("no suitable digital ID supplied");
	    }


	    System.out.println("       Master pw: " +
		((plop.pcos_get_number(doc, "encrypt/master") != 0) ? "yes" : "no"));

	    System.out.println("         User pw: " +
		((plop.pcos_get_number(doc, "encrypt/user") != 0) ? "yes" : "no"));

	    System.out.println("    Text copying: " +
		((plop.pcos_get_number(doc, "encrypt/nocopy") != 0) ? "no" : "yes"));

	    System.out.println("      Linearized: " +
		((plop.pcos_get_number(doc, "linearized") != 0) ? "yes" : "no"));

	    if (pcosmode == 0)
	    {
		System.out.println(
		    "    Minimum mode: no more information available\n");
		plop.delete();
		System.exit(0);
	    }

	    /* --------- more details (requires at least user password) */

	    System.out.println("    PDF/X status: " + plop.pcos_get_string(doc, "pdfx"));

	    System.out.println("    PDF/A status: " + plop.pcos_get_string(doc, "pdfa"));
	    System.out.println("   PDF/UA status: " + plop.pcos_get_string(doc, "pdfua"));
	    System.out.println("   PDF/VT status: " + plop.pcos_get_string(doc, "pdfvt"));

	    boolean xfa_present =
            !plop.pcos_get_string(doc, "type:/Root/AcroForm/XFA").equals("null");
        System.out.println("        XFA data: " + (xfa_present ? "yes" : "no"));

	    System.out.println("      Tagged PDF: " +
		((plop.pcos_get_number(doc, "tagged") != 0) ? "yes" : "no") + "\n");

            /* --------- analyze digital signatures */

            count = (int) plop.pcos_get_number(doc, "length:signaturefields");

            for (i=0; i < count; i++)
            {
                String sigtype;
                String sigfields;

                sigfields = "signaturefields[" + i + "]/fullname";
                System.out.print("  Sig. field #" + i+1 + ": '"
                    + plop.pcos_get_string(doc, sigfields) + "',");

                sigfields = "signaturefields[" + i + "]/visible";
                System.out.print(" "
                    + ((int)plop.pcos_get_number(doc, sigfields) != 0? "visible" : "invisible"));

                sigfields = "signaturefields[" + i + "]/sigtype";
                sigtype = plop.pcos_get_string(doc, sigfields);

                if (sigtype.equals("none"))
                {
                    System.out.print(", unsigned\n");
                    continue;
                }

                System.out.print(" " + sigtype + " signature");

                if (sigtype.equals("certification"))
                {
                    sigfields = "signaturefields[" + i + "]/permissions";
                    System.out.print(" (" + plop.pcos_get_string(doc, sigfields) + " allowed)");
                }

                sigfields = "signaturefields[" + i + "]/cades";
                if ((int)plop.pcos_get_number(doc, sigfields) != 0)
                    System.out.print(", CAdES");

                System.out.print("\n");
            }
            if (count > 0)
            {
                System.out.print("No. of revisions: "
                    + (int) plop.pcos_get_number(doc, "revisions") + "\n\n");
            }

            System.out.print("  Reader-enabled: "
                + plop.pcos_get_string(doc, "usagerights") + "\n");

	    System.out.println("    No. of pages: " +
		(int) plop.pcos_get_number(doc, "length:pages"));

	    System.out.println("     Page 1 size: width=" +
		 (int) plop.pcos_get_number(doc, "pages[0]/width") + ", height=" +
		 (int) plop.pcos_get_number(doc, "pages[0]/height"));

	    count = (int) plop.pcos_get_number(doc, "length:fonts");
	    System.out.println("    No. of fonts: " +  count);

	    for (i=0; i < count; i++)
	    {
		String fonts;

		fonts = "fonts[" + i + "]/embedded";
		if (plop.pcos_get_number(doc, fonts) != 0)
		    System.out.print("embedded ");
		else
		    System.out.print("unembedded ");

		fonts = "fonts[" + i + "]/type";
		System.out.print(plop.pcos_get_string(doc, fonts) + " font ");
		fonts = "fonts[" + i + "]/name";
		System.out.println(plop.pcos_get_string(doc, fonts));
	    }

	    System.out.println();

	    plainmetadata = plop.pcos_get_number(doc, "encrypt/plainmetadata") != 0;

	    if (pcosmode == 1 && !plainmetadata &&
		   plop.pcos_get_number(doc, "encrypt/nocopy") != 0)
	    {
		System.out.print(
		    "Restricted mode: no more information available");
		plop.delete();
		System.exit(0);
	    }

	    /* ----- document info keys and XMP metadata (requires master pw) */

	    count = (int) plop.pcos_get_number(doc, "length:/Info");

	    for (i=0; i < count; i++)
	    {
		String info;
		String key;
		int len;

		info = "type:/Info[" + i + "]";
		objtype = plop.pcos_get_string(doc, info);

		info = "/Info[" + i + "].key";
		key = plop.pcos_get_string(doc, info);
		len = 16 - key.length();
		while (len-- > 0) System.out.print(" ");

		System.out.print(key + ": ");

		/* Info entries can be stored as string or name objects */
		if (objtype.equals("name") || objtype.equals("string"))
		{
		    info = "/Info[" + i + "]";
		    System.out.println("'" + plop.pcos_get_string(doc, info) + "'");
		}
		else
		{
		    info = "type:/Info[" + i + "]";
		    System.out.println("(" + plop.pcos_get_string(doc, info) +
			 " object)");
		}
	    }

	    System.out.print("\n" + "   XMP meta data: ");

	    objtype = plop.pcos_get_string(doc, "type:/Root/Metadata");
	    if (objtype.equals("stream"))
	    {
		byte[] contents;
		int len;

		contents = plop.pcos_get_stream(doc, "", "/Root/Metadata");
		System.out.print(contents.length + " bytes ");
		System.out.println("");
	    }
	    else
	    {
		System.out.println("not present");
	    }

	    plop.close_document(doc, "");
	}

	catch (PLOPException e) {
            System.err.println("PLOP exception occurred in dumper:");
            System.err.println("[" + e.get_errnum() + "] " + e.get_apiname() +
                            ": " + e.get_errmsg());
        } catch (Exception e) {
            System.err.println(e);
        } finally {
            if (plop != null) {
                plop.delete();
            }
        }

	System.exit(0);
    }
}
